from django.core.management import call_command
from django.core.management.base import BaseCommand
from pimpmytheme.folder_management import create_folders
from pimpmytheme.utils import get_lookup_class


class Command(BaseCommand):

    def handle(self, *args, **options):
        objects = get_lookup_class().objects.all()

        for elem in objects:
            create_folders(elem)
        call_command('collectstatic', interactive=False)



import os
from django.contrib.staticfiles.finders import AppDirectoriesFinder
from django.core.files.storage import FileSystemStorage
from django.conf import settings

BASE_FOLDER = settings.PIMPMYTHEME_FOLDER
project_name = settings.SETTINGS_MODULE.split(".")[0]


class CustomStaticStorage(FileSystemStorage):
    """
    A file system storage backend that takes an app module and works
    for the ``static`` directory of it.
    """
    prefix = None
    source_dir = 'static'

    def __init__(self, app, *args, **kwargs):
        """
        Returns a static file storage if available in the given app.
        """
        # app is the actual app module
        location = os.path.join(
            BASE_FOLDER, project_name
            )
        super(CustomStaticStorage, self).__init__(location, *args, **kwargs)


class CustomFinder(AppDirectoriesFinder):
    storage_class = CustomStaticStorage



from django import template
from django.conf import settings
from pimpmytheme.utils import get_lookup_class
from django.utils.safestring import mark_safe
from django.contrib.staticfiles import finders
from django.templatetags.static import static
register = template.Library()
project_name = settings.SETTINGS_MODULE.split(".")[0]


def pimp(context, file_type, filename=None):

    if filename is None:
        filename = ""
    lookup = get_lookup_class().objects.get_current()

    # lookup is not mandatory, maybe we do not have current item right now.
    if not lookup:
        return '#'

    url = "/".join(
        [getattr(lookup, settings.CUSTOM_THEME_LOOKUP_ATTR),
         "static", file_type])
    url = static("".join([url, "/", filename]))
    return url


@register.simple_tag(takes_context=True)
def pimp_css(context, filename=None, css_type=None):
    if css_type is None:
        css_type = "css"
    if pimp_exists(context, "css", filename=filename) is None:
        return ""
    else:
        return mark_safe("""<link rel="stylesheet" href="{}" type="text/{}"
        media="screen" />""".format(pimp(context, "css", filename), css_type))


@register.simple_tag(takes_context=True)
def pimp_js(context, filename=None):
    if pimp_exists(context, "js", filename=filename) is None:
        return ""
    else:
        return mark_safe(
            """<script type="text/javascript" src="{}"></script>""".format(
                pimp(context, "js", filename)))


@register.simple_tag(takes_context=True)
def pimp_img(context, filename=None):
    if pimp_exists(context, "img", filename=filename) is None:
        return ""
    else:
        return mark_safe(
            """<img src="{}" />""".format(
                pimp(context, "img", filename)))


def pimp_exists(context, filetype, filename=None):
    if filename is None:
        filename = ""
    lookup = get_lookup_class().objects.get_current()

    # lookup is not mandatory, maybe we do not have current item right now.
    if not lookup:
        return

    path = "/".join(
        [getattr(lookup, settings.CUSTOM_THEME_LOOKUP_ATTR),
         "static", filetype, filename])
    if finders.find(path) is None:
        return
    return pimp(context, filetype, filename)



# -*- coding: utf-8 -*-
import os

from optparse import make_option

from django.core.management.base import BaseCommand
from django.core.management.base import CommandError

from django.conf import settings


PIMPMYTHEME_FOLDER_SETTINGS_ERROR = """To target the expected theme folder you
need to speficy it in your current django settings with the PIMPMYTHEME_FOLDER
variable or in your command line as bellow:

    python manage.py update_themefolder_from_git --folder <~/some/where>

"""


PIMPMYTHEME_GIT_REPOSITORY_SETTINGS_ERROR = """To clone or update a git
repository of themes you need to speficy it in your current django settings
with the PIMPMYTHEME_GIT_REPOSITORY or in your command line as bellow:

    python manage.py update_themefolder_from_git --git_repository git@github.com:<you>/<your_folder>.git

"""  # noqa


GIT_IMPORT_ERROR = """To update pimpmytheme folder from a git repository you
need to install the `gitpython` library, for example be typing the following
command:

    pip install gitpython

"""


class Command(BaseCommand):

    args = "[--folder] [--git_repository]"

    help = """Clones or updates pimpmytheme folder."""

    option_list = BaseCommand.option_list + (
        make_option(
            '--folder',
            action='store',
            dest='folder',
            help='Overrides PIMPMYTHEME_FOLDER settings value.'
        ),
        make_option(
            '--git_repository',
            action='store',
            dest='git_repository',
            help='Overrides PIMPMYTHEME_GIT_REPOSITORY settings value.'
        ),
    )

    def handle(self, *args, **options):

        # get folder where to clone from command line or settings
        folder = options.get('folder')\
            or getattr(settings, 'PIMPMYTHEME_FOLDER', None)

        # not set
        if not folder:
            raise CommandError(PIMPMYTHEME_FOLDER_SETTINGS_ERROR)

        # get git repository to clone or update from command line or settings
        git_repository = options.get('git_repository')\
            or getattr(settings, 'PIMPMYTHEME_GIT_REPOSITORY', None)

        # not set
        if not git_repository:
            raise CommandError(PIMPMYTHEME_GIT_REPOSITORY_SETTINGS_ERROR)

        # check python git lib requirement
        try:
            import git
        except ImportError:
            raise CommandError(GIT_IMPORT_ERROR)

        # git clone
        try:
            os.makedirs(folder)
            git.Git().clone(git_repository, folder)

        # git update if folder already exist
        except OSError:
            repo = git.Repo(folder)
            repo.git.pull()



"""
Django settings for example project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import sys
BASE_DIR = os.path.dirname(os.path.dirname(__file__))
sys.path.append(os.path.join(BASE_DIR, ".."))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '#^qtdm4ad9_44k+pf+2^ecrm(w9j@w(+s(^e$@s8l=zq%pqtwl'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'pimpmytheme',
    'subapp',
    'compressor',
    'django_nose'
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'example.urls'

WSGI_APPLICATION = 'example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/


STATIC_ROOT = os.path.join(BASE_DIR, "example", "static")
STATIC_URL = '/static/'
from django.conf import global_settings
TEMPLATE_CONTEXT_PROCESSORS = global_settings.TEMPLATE_CONTEXT_PROCESSORS + (
    "pimpmytheme.context_processors.get_site",
)

SITE_ID = 1

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder',
    )

COMPRESS_PRECOMPILERS = (('text/less', 'lessc {infile} {outfile}'),)

TEMPLATE_LOADERS = (
    'pimpmytheme.template_loader.Loader',
    'django.template.loaders.app_directories.Loader',
    )

STATICFILES_FINDERS = (
    "pimpmytheme.static_finder.CustomFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "compressor.finders.CompressorFinder"
    )

NOSE_ARGS = [
    '--with-coverage',
    '--cover-package=pimpmytheme',
    '--verbosity=3',
    '--nocapture'
    ]

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
CUSTOM_THEME_LOOKUP_OBJECT = "django.contrib.sites.models.Site"
CUSTOM_THEME_LOOKUP_ATTR = "name"
PIMPMYTHEME_FOLDER = os.path.join(BASE_DIR, "pimp_theme")



import importlib
from django.conf import settings


def get_lookup_class():
    module_string = '.'.join(
        settings.CUSTOM_THEME_LOOKUP_OBJECT.split('.')[:-1])
    klass_string = settings.CUSTOM_THEME_LOOKUP_OBJECT.split('.')[-1]
    klass = getattr(importlib.import_module(module_string), klass_string)
    return klass



import os
from django.conf import settings

BASE_FOLDER = settings.PIMPMYTHEME_FOLDER
project_name = settings.SETTINGS_MODULE.split(".")[0]


def create_folders(instance):
    curent_dir = os.path.join(BASE_FOLDER, project_name,
                              getattr(instance,
                                      settings.CUSTOM_THEME_LOOKUP_ATTR),
                              "static", "css")
    if not os.path.exists(curent_dir):
        os.makedirs(curent_dir)
    custom_less = os.path.join(curent_dir, "custom.less")
    if not os.path.exists(custom_less):
        f = open(custom_less, "w")
        f.close()



import os
from django.template.loaders.app_directories import Loader
from django.utils._os import safe_join
from django.conf import settings
from .utils import get_lookup_class
BASE_FOLDER = settings.PIMPMYTHEME_FOLDER
project_name = settings.SETTINGS_MODULE.split(".")[0]


class Loader(Loader):
    is_usable = True

    def get_template_sources(self, template_name, template_dirs=None):

        lookup = get_lookup_class().objects.get_current()

        # maybe we do not have current lookup so we just do not try to handle
        # template dir now
        if not lookup:
            return []

        if not template_dirs:
            template_dir = os.path.join(
                BASE_FOLDER,
                project_name,
                getattr(lookup,
                        settings.CUSTOM_THEME_LOOKUP_ATTR),
                'templates')

        try:
            return [safe_join(template_dir, template_name)]
        except UnicodeDecodeError:
            # The template dir name was a bytestring that wasn't
            # valid UTF-8.
            raise
        except ValueError:
            # The joined path was located outside of template_dir.
            pass



from django.utils.functional import SimpleLazyObject
from django.contrib.sites.models import get_current_site


def get_site(request):
    return {
        'site': SimpleLazyObject(lambda: get_current_site(request)),
        }



#!/usr/bin/env python
#-*- coding: utf-8 -*-
from os.path import join, dirname, abspath
import sys

PY2 = sys.version_info[0] == 2
ROOT = dirname(abspath(__file__))

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup  # NOQA


def read_relative_file(filename):
    """Returns contents of the given file, whose path is supposed relative
    to this module."""
    with open(join(ROOT, filename)) as f:
        return f.read()

NAME = 'django-pimpmytheme'
DESCRIPTION = """Customise theme (css and template) on a per
user/client/whatever basis"""
REQUIREMENTS = [
    'django',
    'django-compressor',
]
__VERSION__ = read_relative_file('VERSION').strip()


params = dict(
    name=NAME,
    description=DESCRIPTION,
    packages=['pimpmytheme'],
    version=__VERSION__,
    long_description=read_relative_file('README.rst'),
    author='Yohann Gabory',
    author_email='yohann.gabory@novapost.fr',
    url='https://github.com/novapost/pimpmytheme',
    license='MIT License',
    include_package_data=True,
    install_requires=REQUIREMENTS,
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
)

if __name__ == '__main__':
    setup(**params)



import os
import shutil

from django.test import TestCase
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.management import call_command
from django.core.management.base import CommandError

from pimpmytheme.management.commands import update_themefolder_from_git as ufg

from pimpmytheme.templatetags.pimptheme import pimp
from pimpmytheme.templatetags.pimptheme import pimp_exists


class NoneLookupManager(object):

    def get_current(self):
        return None


class NoneLookup(object):

    objects = NoneLookupManager()


project_name = settings.SETTINGS_MODULE.split(".")[0]
BASE_FOLDER = settings.PIMPMYTHEME_FOLDER


class SiteTestCase(TestCase):
    """
    Ensure that, when a site is added, the folders are created too
    """

    def tearDown(self):
        settings.PIMPMYTHEME_FOLDER = BASE_FOLDER

    def test_folder_creation(self):
        Site.objects.create(name="something",
                            domain="something.com")
        call_command("create_folders")
        current_dir = os.path.join(BASE_FOLDER, project_name,
                                   "something", "static", "css")
        self.assertTrue(os.path.exists(current_dir))
        custom_less = os.path.join(current_dir, "custom.less")
        self.assertTrue(os.path.exists(custom_less))
        shutil.rmtree(os.path.join(BASE_FOLDER, project_name, "something"))
        self.assertFalse(os.path.exists(current_dir))

    def test_update_themefolder_from_git_command(self):
        # invalidate PIMPMYTHEME_FOLDER variable
        settings.PIMPMYTHEME_FOLDER = None

        # no repo settings
        self.assertRaisesRegexp(CommandError,
                                ufg.PIMPMYTHEME_FOLDER_SETTINGS_ERROR,
                                call_command,
                                'update_themefolder_from_git')

        # no repo settings
        self.assertRaisesRegexp(CommandError,
                                ufg.PIMPMYTHEME_GIT_REPOSITORY_SETTINGS_ERROR,
                                call_command,
                                'update_themefolder_from_git', folder='.')

        # dummy repo
        try:
            import git
            self.assertRaisesRegexp(git.GitCommandError,
                                    'returned exit status 1',
                                    call_command,
                                    'update_themefolder_from_git', folder='.',
                                    git_repository='dummy')
        except ImportError:
            self.assertRaisesRegexp(CommandError,
                                    ufg.GIT_IMPORT_ERROR,
                                    call_command,
                                    'update_themefolder_from_git', folder='.',
                                    git_repository='dummy')


class TemplatetagsTestCase(TestCase):

    def tearDown(self):
        # restore standard behaviour
        settings.CUSTOM_THEME_LOOKUP_OBJECT = \
            'django.contrib.sites.models.Site'

    def test_pimp(self):
        res = pimp({}, 'css')
        self.assertEqual(res, '/static/example.com/static/css/')

        res = pimp({}, 'css', filename='custom.less')
        self.assertEqual(res, '/static/example.com/static/css/custom.less')

        # if no current item
        settings.CUSTOM_THEME_LOOKUP_OBJECT = 'pimpmytheme.tests.NoneLookup'

        res = pimp({}, 'css')
        self.assertEqual(res, '#')

        res = pimp({}, 'css', filename='custom.less')
        self.assertEqual(res, '#')

    def test_pimp_exists(self):
        res = pimp_exists({}, 'css', filename=None)
        self.assertEqual(res, '/static/example.com/static/css/')

        res = pimp_exists({}, 'css', filename='custom.less')
        self.assertEqual(res, '/static/example.com/static/css/custom.less')

        # if no current item
        settings.CUSTOM_THEME_LOOKUP_OBJECT = 'pimpmytheme.tests.NoneLookup'

        res = pimp_exists({}, 'css', filename=None)
        self.assertIsNone(res)

        res = pimp_exists({}, 'css', filename='custom.less')
        self.assertIsNone(res)
